@@ -77,7 +77,7 @@ struct path Cur_Path = {
 
 /*
  * dir_remove queues struct dirs to be freed here. We can't just delete them immeidately
- * eg in dircache_search_by_id, because a caller somewhere up the stack might be
+ * e.g. in dircache_search_by_id, because a caller somewhere up the stack might be
  * referencing it.
  * So instead:
  * - we mark it as invalid by setting d_did to CNID_INVALID (ie 0)
@@ -370,7 +370,7 @@ static int cname_mtouname(const struct vol *vol, struct dir *dir, struct path *r
  * 1. move cwd into parent dir (we're often already there, but not always)
  * 2. set struct path to the dirname
  * 3. in case of
- *    AFPERR_ACCESS: the dir is there, we just cant chdir into it
+ *    AFPERR_ACCESS: the dir is there, we just can't chdir into it
  *    AFPERR_NOOBJ: the dir was there when we stated it in cname, so we have a race
  *                  4. indicate there's no dir for this path
  *                  5. remove the dir
@@ -718,7 +718,6 @@ exit:
     return ret;
 }
 
-#define ENUMVETO "./../Network Trash Folder/TheVolumeSettingsFolder/TheFindByContentFolder/:2eDS_Store/Contents/Desktop Folder/Trash/Benutzer/"
 
 /*!
  * @brief Construct struct dir
@@ -1048,8 +1047,8 @@ int dir_modify(const struct vol *vol,
  * @brief Resolve a catalog node name path
  *
  * 1. Evaluate path type
- * 2. Move to start dir, if we cant, it might eg because of EACCES, build
- *    path from dirname, so eg getdirparams has sth it can chew on. curdir
+ * 2. Move to start dir, if we can't, it might be e.g. because of EACCES, build
+ *    path from dirname, so e.g. getdirparams has sth it can chew on. curdir
  *    is dir parent then. All this is done in path_from_dir().
  * 3. Parse next cnode name in path, cases:
  * 4.   single "\0" -> do nothing
@@ -1165,7 +1164,7 @@ struct path *cname(struct vol *vol, struct dir *dir, char **cpath)
             return NULL;
         }
 
-        LOG(log_maxdebug, logtype_afpd, "came('%s'): {node: '%s}", cfrombstr(dir->d_fullpath), ret.u_name);
+        LOG(log_maxdebug, logtype_afpd, "cname('%s'): {node: '%s}", cfrombstr(dir->d_fullpath), ret.u_name);
 
         /* Prevent access to our special folders like .AppleDouble */
         if (check_name(vol, ret.u_name)) {
@@ -1195,7 +1194,7 @@ struct path *cname(struct vol *vol, struct dir *dir, char **cpath)
                 return NULL;
         } else {
             /*
-             * CNID != 1, eg. most of the times we take this way.
+             * CNID != 1, e.g. most of the times we take this way.
              * Now check if current path-part is a file or dir:
              * o if it's dir we have to step into it
              * o if it's a file we expect it to be the last part of the requested path
@@ -1216,14 +1215,14 @@ struct path *cname(struct vol *vol, struct dir *dir, char **cpath)
                  * this will terminate clean in while (1) because len == 0,
                  * probably afp_createfile|dir
                  */
-                LOG(log_maxdebug, logtype_afpd, "came('%s'): {leave-cnode ENOENT (possile create request): '%s'}",
+                LOG(log_maxdebug, logtype_afpd, "cname('%s'): {leave-cnode ENOENT (probably create request): '%s'}",
                     cfrombstr(dir->d_fullpath), ret.u_name);
                 continue; /* 10 */
             }
 
             switch (ret.st.st_mode & S_IFMT) {
             case S_IFREG: /* 11 */
-                LOG(log_debug, logtype_afpd, "came('%s'): {file: '%s'}",
+                LOG(log_debug, logtype_afpd, "cname('%s'): {file: '%s'}",
                     cfrombstr(dir->d_fullpath), ret.u_name);
                 if (len > 0) {
                     /* it wasn't the last part, so we have a bogus path request */
@@ -1232,10 +1231,10 @@ struct path *cname(struct vol *vol, struct dir *dir, char **cpath)
                 }
                 continue; /* continues while loop */
             case S_IFLNK: /* 12 */
-                LOG(log_debug, logtype_afpd, "came('%s'): {link: '%s'}",
+                LOG(log_debug, logtype_afpd, "cname('%s'): {link: '%s'}",
                     cfrombstr(dir->d_fullpath), ret.u_name);
                 if (len > 0) {
-                    LOG(log_warning, logtype_afpd, "came('%s'): {symlinked dir: '%s'}",
+                    LOG(log_warning, logtype_afpd, "cname('%s'): {symlinked dir: '%s'}",
                         cfrombstr(dir->d_fullpath), ret.u_name);
                     afp_errno = AFPERR_PARAM;
                     return NULL;
@@ -1426,6 +1425,7 @@ int getdirparams(const AFPObj *obj,
     struct maccess  ma;
     struct adouble  ad;
     char        *data, *l_nameoff = NULL, *utf_nameoff = NULL;
+    char        *ade = NULL;
     int         bit = 0, isad = 0;
     uint32_t           aint;
     uint16_t       ashort;
@@ -1519,8 +1519,8 @@ int getdirparams(const AFPObj *obj,
             break;
 
         case DIRPBIT_FINFO :
-            if ( isad ) {
-                memcpy( data, ad_entry( &ad, ADEID_FINDERI ), 32 );
+            if ( isad && (ade = ad_entry(&ad, ADEID_FINDERI)) != NULL) {
+                memcpy( data, ade, 32 );
             } else { /* no appledouble */
                 memset( data, 0, 32 );
                 /* dot files are by default visible */
@@ -1734,7 +1734,7 @@ int afp_setdirparams(AFPObj *obj, char *ibuf, size_t ibuflen _U_, char *rbuf _U_
 }
 
 /*
- * assume path == '\0' eg. it's a directory in canonical form
+ * assume path == '\0' e.g. it's a directory in canonical form
  */
 int setdirparams(struct vol *vol, struct path *path, uint16_t d_bitmap, char *buf )
 {
@@ -1744,6 +1744,7 @@ int setdirparams(struct vol *vol, struct path *path, uint16_t d_bitmap, char *bu
     struct timeval      tv;
 
     char                *upath;
+    char                *ade = NULL;
     struct dir          *dir;
     int         bit, isad = 0;
     int                 cdate, bdate;
@@ -1898,7 +1899,7 @@ int setdirparams(struct vol *vol, struct path *path, uint16_t d_bitmap, char *bu
             }
             break;
         case DIRPBIT_FINFO :
-            if (isad) {
+            if (isad && (ade = ad_entry(&ad, ADEID_FINDERI)) != NULL) {
                 /* Fixes #2802236 */
                 uint16_t fflags;
                 memcpy(&fflags, finder_buf + FINDERINFO_FRFLAGOFF, sizeof(uint16_t));
@@ -1915,10 +1916,10 @@ int setdirparams(struct vol *vol, struct path *path, uint16_t d_bitmap, char *bu
                      * behavior one sees when mounting above another mount
                      * point.
                      */
-                    memcpy( ad_entry( &ad, ADEID_FINDERI ), finder_buf, 10 );
-                    memcpy( ad_entry( &ad, ADEID_FINDERI ) + 14, finder_buf + 14, 18 );
+                    memcpy( ade, finder_buf, 10 );
+                    memcpy( ade + 14, finder_buf + 14, 18 );
                 } else {
-                    memcpy( ad_entry( &ad, ADEID_FINDERI ), finder_buf, 32 );
+                    memcpy( ade, finder_buf, 32 );
                 }
             }
             break;
@@ -2506,6 +2507,9 @@ int afp_mapname(AFPObj *obj, char *ibuf, size_t ibuflen _U_, char *rbuf, size_t
         return( AFPERR_PARAM );
     }
 
+    if (len >= ibuflen - 1)
+        return AFPERR_PARAM;
+
     ibuf[ len ] = '\0';
 
     if ( len == 0 )
