@@ -1,6 +1,7 @@
 /*
+ *
  * Copyright (c) 1990,1993 Regents of The University of Michigan.
- * Copyright (c) 1999 Adrian Sun (asun@u.washington.edu)
+ * Copyright (c) 1999 Adrian Sun (asun@u.washington.edu) 
  * All Rights Reserved.  See COPYRIGHT.
  */
 
@@ -14,7 +15,9 @@
 #include <string.h>
 #include <atalk/logger.h>
 
+#ifdef HAVE_UNISTD_H
 #include <unistd.h>
+#endif /* HAVE_UNISTD_H */
 #include <errno.h>
 #ifdef HAVE_SECURITY_PAM_APPL_H
 #include <security/pam_appl.h>
@@ -22,7 +25,7 @@
 #ifdef HAVE_PAM_PAM_APPL_H
 #include <pam/pam_appl.h>
 #endif
-
+#include <arpa/inet.h>
 
 #if defined(GNUTLS_DHX)
 #include <gnutls/openssl.h>
@@ -31,6 +34,7 @@
 #include <openssl/dh.h>
 #include <openssl/cast.h>
 #include <openssl/err.h>
+#include "openssl_compat.h"
 #else /* OPENSSL_DHX */
 #include <bn.h>
 #include <dh.h>
@@ -54,29 +58,29 @@
 /* the secret key */
 static CAST_KEY castkey;
 static struct passwd *dhxpwd;
-static u_int8_t randbuf[KEYSIZE];
+static uint8_t randbuf[KEYSIZE];
 
 /* diffie-hellman bits */
 static unsigned char msg2_iv[] = "CJalbert";
 static unsigned char msg3_iv[] = "LWallace";
-static const u_int8_t p[] = {0xBA, 0x28, 0x73, 0xDF, 0xB0, 0x60, 0x57, 0xD4,
+static const uint8_t p[] = {0xBA, 0x28, 0x73, 0xDF, 0xB0, 0x60, 0x57, 0xD4,
 			     0x3F, 0x20, 0x24, 0x74, 0x4C, 0xEE, 0xE7, 0x5B};
-static const u_int8_t g = 0x07;
+static const uint8_t g = 0x07;
 
 
 /* Static variables used to communicate between the conversation function
  * and the server_login function
  */
 static pam_handle_t *pamh = NULL; 
-static unsigned char *PAM_username;
-static unsigned char *PAM_password;
+static char *PAM_username;
+static char *PAM_password;
 
 /* PAM conversation function
  * Here we assume (for now, at least) that echo on means login name, and
  * echo off means password.
  */
 static int PAM_conv (int num_msg,
-#if !defined(__svr4__)
+#ifdef LINUX
                      const struct pam_message **msg,
 #else
                      struct pam_message **msg,
@@ -114,7 +118,7 @@ static int PAM_conv (int num_msg,
 
     switch (msg[count]->msg_style) {
     case PAM_PROMPT_ECHO_ON:
-      if (!(string = COPY_STRING((const char *)PAM_username))) {
+      if (!(string = COPY_STRING(PAM_username))) {
     /* Log Entry */
            LOG(log_info, logtype_uams, "uams_dhx_pam.c :PAM: username failure -- %s",
 		  strerror(errno));
@@ -123,7 +127,7 @@ static int PAM_conv (int num_msg,
       }
       break;
     case PAM_PROMPT_ECHO_OFF:
-      if (!(string = COPY_STRING((const char *)PAM_password))) {
+      if (!(string = COPY_STRING(PAM_password))) {
     /* Log Entry */
            LOG(log_info, logtype_uams, "uams_dhx_pam.c :PAM: passwd failure: --: %s",
 		  strerror(errno));
@@ -184,17 +188,17 @@ static struct pam_conv PAM_conversation = {
 };
 
 
-static int dhx_setup(void *obj, unsigned char *ibuf, size_t ibuflen _U_, 
-		     unsigned char *rbuf, size_t *rbuflen)
+static int dhx_setup(void *obj, char *ibuf, size_t ibuflen _U_, 
+		     char *rbuf, size_t *rbuflen)
 {
-    u_int16_t sessid;
+    uint16_t sessid;
     size_t i;
     BIGNUM *bn, *gbn, *pbn;
     const BIGNUM *pub_key;
     DH *dh;
 
     /* get the client's public key */
-    if (!(bn = BN_bin2bn(ibuf, KEYSIZE, NULL))) {
+    if (!(bn = BN_bin2bn((unsigned char *)ibuf, KEYSIZE, NULL))) {
     /* Log Entry */
            LOG(log_info, logtype_uams, "uams_dhx_pam.c :PAM No Public Key -- %s",
 		  strerror(errno));
@@ -238,7 +242,7 @@ static int dhx_setup(void *obj, unsigned char *ibuf, size_t ibuflen _U_,
       BN_free(pbn);
       BN_free(gbn);
     /* Log Entry */
-      LOG(log_info, logtype_uams, "uams_dhx_pam.c :PAM DH_set0_pqg() mysteriously failed  -- %s", strerror(errno));
+        LOG(log_info, logtype_uams, "uams_dhx_pam.c :PAM DH_set0_pqg() mysteriously failed  -- %s", strerror(errno));
     /* Log Entry */
       goto pam_fail;
     }
@@ -257,10 +261,10 @@ static int dhx_setup(void *obj, unsigned char *ibuf, size_t ibuflen _U_,
 	ERR_free_strings();
 	goto pam_fail;
     }
-    DH_get0_key(dh, &pub_key, NULL);
+	DH_get0_key(dh, &pub_key, NULL);
     if (BN_num_bytes(pub_key) > KEYSIZE) {
-	LOG(log_info, logtype_uams, "uams_dhx_pam.c :PAM: Err Generating Key -- Not enough Space? -- %s", strerror(errno));
-	goto pam_fail;
+      LOG(log_info, logtype_uams, "uams_dhx_pam.c :PAM: Err Generating Key -- Not enough Space? -- %s", strerror(errno));
+      goto pam_fail;
     }
 
     /* figure out the key. store the key in rbuf for now. */
@@ -276,7 +280,7 @@ static int dhx_setup(void *obj, unsigned char *ibuf, size_t ibuflen _U_,
     *rbuflen += sizeof(sessid);
     
     /* public key */
-    BN_bn2bin(pub_key, (unsigned char *)rbuf); 
+    BN_bn2bin(pub_key, (unsigned char *)rbuf);
     rbuf += KEYSIZE;
     *rbuflen += KEYSIZE;
 
@@ -310,7 +314,7 @@ static int dhx_setup(void *obj, unsigned char *ibuf, size_t ibuflen _U_,
 #endif /* 0 */
 
     /* encrypt using cast */
-    CAST_cbc_encrypt(rbuf, rbuf, CRYPTBUFLEN, &castkey, msg2_iv, 
+    CAST_cbc_encrypt((unsigned char *)rbuf, (unsigned char *)rbuf, CRYPTBUFLEN, &castkey, msg2_iv, 
 		     CAST_ENCRYPT);
     *rbuflen += CRYPTBUFLEN;
     BN_free(bn);
@@ -328,11 +332,11 @@ pam_fail:
 }
 
 /* -------------------------------- */
-static int login(void *obj, unsigned char *username, int ulen,
-	struct passwd **uam_pwd _U_, unsigned char *ibuf, size_t ibuflen,
-	unsigned char *rbuf, size_t *rbuflen)
+static int login(void *obj, char *username, int ulen,  struct passwd **uam_pwd _U_,
+		     char *ibuf, size_t ibuflen,
+		     char *rbuf, size_t *rbuflen)
 {
-    if (( dhxpwd = uam_getname(obj, (char *)username, ulen)) == NULL ) {
+    if (( dhxpwd = uam_getname(obj, username, ulen)) == NULL ) {
         LOG(log_info, logtype_uams, "uams_dhx_pam.c: unknown username [%s]", username);
         return AFPERR_NOTAUTH;
     }
@@ -346,10 +350,10 @@ static int login(void *obj, unsigned char *username, int ulen,
 /* dhx login: things are done in a slightly bizarre order to avoid
  * having to clean things up if there's an error. */
 static int pam_login(void *obj, struct passwd **uam_pwd,
-		     unsigned char *ibuf, size_t ibuflen,
-		     unsigned char *rbuf, size_t *rbuflen)
+		     char *ibuf, size_t ibuflen,
+		     char *rbuf, size_t *rbuflen)
 {
-    unsigned char *username;
+    char *username;
     size_t len, ulen;
 
     *rbuflen = 0;
@@ -380,12 +384,13 @@ static int pam_login(void *obj, struct passwd **uam_pwd,
 
 /* ----------------------------- */
 static int pam_login_ext(void *obj, char *uname, struct passwd **uam_pwd,
-		     unsigned char *ibuf, size_t ibuflen,
-		     unsigned char *rbuf, size_t *rbuflen)
+		     char *ibuf, size_t ibuflen,
+		     char *rbuf, size_t *rbuflen)
 {
-    unsigned char *username;
-    size_t len, ulen;
-    u_int16_t  temp16;
+    char *username;
+    int len;
+    size_t ulen;
+    uint16_t  temp16;
 
     *rbuflen = 0;
 
@@ -416,12 +421,12 @@ static int pam_login_ext(void *obj, char *uname, struct passwd **uam_pwd,
 /* -------------------------------- */
 
 static int pam_logincont(void *obj, struct passwd **uam_pwd,
-			 unsigned char *ibuf, size_t ibuflen _U_, 
-			 unsigned char *rbuf, size_t *rbuflen)
+			 char *ibuf, size_t ibuflen _U_, 
+			 char *rbuf, size_t *rbuflen)
 {
     const char *hostname;
     BIGNUM *bn1, *bn2, *bn3;
-    u_int16_t sessid;
+    uint16_t sessid;
     int err, PAM_error;
 
     *rbuflen = 0;
@@ -444,13 +449,13 @@ static int pam_logincont(void *obj, struct passwd **uam_pwd,
 	hostname = NULL;
 	}
 
-    CAST_cbc_encrypt(ibuf, rbuf, CRYPT2BUFLEN, &castkey,
+    CAST_cbc_encrypt((unsigned char *)ibuf, (unsigned char *)rbuf, CRYPT2BUFLEN, &castkey,
 		     msg3_iv, CAST_DECRYPT);
     memset(&castkey, 0, sizeof(castkey));
 
     /* check to make sure that the random number is the same. we
      * get sent back an incremented random number. */
-    if (!(bn1 = BN_bin2bn(rbuf, KEYSIZE, NULL)))
+    if (!(bn1 = BN_bin2bn((unsigned char *)rbuf, KEYSIZE, NULL)))
       return AFPERR_PARAM;
 
     if (!(bn2 = BN_bin2bn(randbuf, sizeof(randbuf), NULL))) {
@@ -485,8 +490,8 @@ static int pam_logincont(void *obj, struct passwd **uam_pwd,
     PAM_password = rbuf;
 
     err = AFPERR_NOTAUTH;
-    PAM_error = pam_start("netatalk", (const char *)PAM_username,
-			&PAM_conversation, &pamh);
+    PAM_error = pam_start("netatalk", PAM_username, &PAM_conversation,
+			  &pamh);
     if (PAM_error != PAM_SUCCESS) {
     /* Log Entry */
            LOG(log_info, logtype_uams, "uams_dhx_pam.c :PAM: PAM_Error: %s",
@@ -572,16 +577,16 @@ static void pam_logout(void) {
 
 /* change pw for dhx needs a couple passes to get everything all
  * right. basically, it's like the login/logincont sequence */
-static int pam_changepw(void *obj, unsigned char *username,
-			struct passwd *pwd _U_, unsigned char *ibuf,
-			size_t ibuflen, unsigned char *rbuf, size_t *rbuflen)
+static int pam_changepw(void *obj, char *username,
+			struct passwd *pwd _U_, char *ibuf, size_t ibuflen,
+			char *rbuf, size_t *rbuflen)
 {
     BIGNUM *bn1, *bn2, *bn3;
 
     char *hostname;
     pam_handle_t *lpamh;
     uid_t uid;
-    u_int16_t sessid;
+    uint16_t sessid;
     int PAM_error;
 
     if (ibuflen < sizeof(sessid)) {
@@ -621,13 +626,13 @@ static int pam_changepw(void *obj, unsigned char *username,
     }
 
     /* grab the client's nonce, old password, and new password. */
-    CAST_cbc_encrypt(ibuf, ibuf, CHANGEPWBUFLEN, &castkey,
+    CAST_cbc_encrypt((unsigned char *)ibuf, (unsigned char *)ibuf, CHANGEPWBUFLEN, &castkey,
 		     msg3_iv, CAST_DECRYPT);
     memset(&castkey, 0, sizeof(castkey));
 
     /* check to make sure that the random number is the same. we
      * get sent back an incremented random number. */
-    if (!(bn1 = BN_bin2bn(ibuf, KEYSIZE, NULL))) {
+    if (!(bn1 = BN_bin2bn((unsigned char *)ibuf, KEYSIZE, NULL))) {
     /* Log Entry */
            LOG(log_info, logtype_uams, "uams_dhx_pam.c :PAM: Random Number Not the same or not incremented-- %s",
 		  strerror(errno));
@@ -678,10 +683,12 @@ static int pam_changepw(void *obj, unsigned char *username,
     /* Set these things up for the conv function. the old password
      * is at the end. */
     ibuf += KEYSIZE;
+    if (ibuflen <= PASSWDLEN + PASSWDLEN)
+        return AFPERR_PARAM;
     ibuf[PASSWDLEN + PASSWDLEN] = '\0';
     PAM_password = ibuf + PASSWDLEN;
 
-    PAM_error = pam_start("netatalk", (char *)username, &PAM_conversation,
+    PAM_error = pam_start("netatalk", username, &PAM_conversation,
 			  &lpamh);
     if (PAM_error != PAM_SUCCESS) {
     /* Log Entry */
@@ -708,6 +715,8 @@ static int pam_changepw(void *obj, unsigned char *username,
 
     /* new password */
     PAM_password = ibuf;
+    if (ibuflen <= PASSWDLEN)
+        return AFPERR_PARAM;
     ibuf[PASSWDLEN] = '\0';
 
     /* this really does need to be done as root */
@@ -724,7 +733,7 @@ static int pam_changepw(void *obj, unsigned char *username,
 }
 
 
-static int uam_setup(const char *path)
+static int uam_setup(void *obj, const char *path)
 {
   if (uam_register(UAM_SERVER_LOGIN_EXT, path, "DHCAST128", pam_login, 
 		   pam_logincont, pam_logout, pam_login_ext) < 0)
