@@ -11,20 +11,16 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-
-#ifdef HAVE_STDINT_H
 #include <stdint.h>
-#endif
-
-#ifdef HAVE_GETIFADDRS
-#include <ifaddrs.h>
-#endif
-
 #include <sys/types.h>
 #include <sys/socket.h>
 #include <sys/ioctl.h>
 #include <netinet/in.h>
 #include <netinet/tcp.h>
+#ifdef TRU64
+#include <sys/mbuf.h>
+#include <net/route.h>
+#endif /* TRU64 */
 #include <net/if.h>
 #include <errno.h>
 
@@ -74,28 +70,6 @@ static int getifaces(const int sockfd, char ***list)
 	  *list = new;
       return i;
 
-#elif defined(HAVE_GETIFADDRS)
-    struct ifaddrs *ifa, *a;
-    int i;
-    char **new;
-
-    if (!list)
-	return 0;
-    if (getifaddrs(&ifa) == -1)
-	return 0;
-    for (i = 0, a = ifa; a != NULL; a = a->ifa_next, i++)
-	continue;
-    new = malloc((i + 1) * sizeof(char *));
-    if (new == NULL) {
-	freeifaddrs(ifa);
-	return 0;
-    }
-    for (i = 0, a = ifa; a != NULL; a = a->ifa_next)
-	if (addname(new, &i, a->ifa_name) < 0)
-	    break;
-    freeifaddrs(ifa);
-    *list = new;
-    return i;
 #else
     struct ifconf	ifc;
     struct ifreq	ifrs[ 64 ], *ifr, *nextifr;
